/***********************************************
 *
 * $GAMGI/src/gtk/atom/gamgi_gtk_atom_select.c
 *
 * Copyright (C) 2004 Carlos Pereira
 *
 * Distributed under the terms of the GNU
 * General Public License: $GAMGI/LICENSE
 *
 */

#include "gamgi_engine.h"
#include "gamgi_gtk.h"
#include "gamgi_io.h"
#include "gamgi_math.h"
#include "gamgi_global.h"

#include "gamgi_engine_list.h"
#include "gamgi_engine_dlist.h"
#include "gamgi_gtk_select.h"
#include "gamgi_gtk_dialog.h"
#include "gamgi_mesa_select.h"
#include "gamgi_chem_atom.h"
#include "gamgi_io_token.h"
#include "gamgi_global_selection.h"

/*************************************************
 * local enumerations in GAMGI always start at 0 *
 *************************************************/

enum { STATIC_SCOPE_GAMGI, STATIC_SCOPE_WINDOW, STATIC_SCOPE_LAYER,
STATIC_SCOPE_BEFORE, STATIC_SCOPE_CURRENT };

enum { STATIC_ACTION_INCLUDE, STATIC_ACTION_EXCLUDE };

enum { STATIC_METHOD_PICK, STATIC_METHOD_ELEMENT, STATIC_METHOD_TYPE,
STATIC_METHOD_MASS, STATIC_METHOD_RADIUS, STATIC_METHOD_CHARGE,
STATIC_METHOD_TEMPERATURE, STATIC_METHOD_OCCUPANCY, STATIC_METHOD_X,
STATIC_METHOD_Y, STATIC_METHOD_Z, STATIC_METHOD_R,
STATIC_METHOD_G, STATIC_METHOD_B, STATIC_METHOD_SIZE,
STATIC_METHOD_VARIANCY, STATIC_METHOD_SCALE };

/**********************************************************
 * GAMGI provides an Undo mechanism allowing users to     *
 * recover the list of atoms selected before the current  *
 * one. atoms_old points to this list, which is available *
 * only while the Atom->Select dialog exists.             *
 **********************************************************/

static gamgi_dlist *atoms_old;

static void static_destroy (GtkWidget *widget, void *data)
{
/**********************************************
 * everytime the dialog is removed: free list *
 * Before and link objects in list Current    *
 **********************************************/

gamgi_engine_dlist_remove_all (atoms_old);
gamgi_global_selection_link (gamgi->atoms);
}

static void static_model (gamgi_dlist *dlist, gamgi_window *window)
{
GtkWidget *dialog = window->dialog0;
GtkWidget *treeview;
GtkTreeStore *store;

store = gtk_tree_store_new (2, G_TYPE_STRING, G_TYPE_POINTER);
while (dlist != NULL)
  {
  gamgi_gtk_select_atom (store, NULL, GAMGI_CAST_ATOM dlist->data);
  dlist = dlist->next;
  }

treeview = (GtkWidget *) g_object_get_data (G_OBJECT (dialog), "treeview");
gtk_tree_view_set_model (GTK_TREE_VIEW (treeview), GTK_TREE_MODEL (store));
g_object_unref (store);
}

static gamgi_dlist *static_dlist (int scope, gamgi_window *window)
{
gamgi_dlist *dlist = NULL;

switch (scope)
  {
  case STATIC_SCOPE_GAMGI:
  dlist = gamgi_engine_dlist_atom_gamgi (NULL);
  break;

  case STATIC_SCOPE_WINDOW:
  dlist = gamgi_engine_dlist_atom_window (window, NULL);
  break;

  case STATIC_SCOPE_LAYER:
  dlist = gamgi_engine_dlist_atom_layer (window->layer, NULL);
  break;

  case STATIC_SCOPE_BEFORE:
  dlist = atoms_old;
  break;

  case STATIC_SCOPE_CURRENT:
  dlist = gamgi->atoms;
  break;
  }

return dlist;
}

static void static_scope (GtkWidget *widget, void *data)
{
gamgi_window *window = GAMGI_CAST_WINDOW data;
gamgi_dlist *dlist;
GtkWidget *dialog = window->dialog0;
GtkWidget *combo;
int scope;

/***********************
 * get scope combo box *
 ***********************/

combo = (GtkWidget *) g_object_get_data (G_OBJECT (dialog), "combo_scope");
scope = gtk_combo_box_get_active (GTK_COMBO_BOX (combo));

/*********************************************
 * update model and remove unnecessary lists *
 *********************************************/

dlist = static_dlist (scope, window);
static_model (dlist, window);
if (scope != STATIC_SCOPE_BEFORE && scope != STATIC_SCOPE_CURRENT)
  gamgi_engine_dlist_remove_all (dlist);
}

static void static_reset (gamgi_window *window)
{
GtkWidget *dialog = window->dialog0;
GtkWidget *combo;
GtkWidget *entry;

combo = (GtkWidget *) g_object_get_data (G_OBJECT (dialog), "combo_scope");
gtk_combo_box_set_active (GTK_COMBO_BOX (combo), STATIC_SCOPE_CURRENT);

combo = (GtkWidget *) g_object_get_data (G_OBJECT (dialog), "combo_action");
gtk_combo_box_set_active (GTK_COMBO_BOX (combo), STATIC_ACTION_INCLUDE);

combo = (GtkWidget *) g_object_get_data (G_OBJECT (dialog), "combo_method");
gtk_combo_box_set_active (GTK_COMBO_BOX (combo), STATIC_METHOD_PICK);

entry = (GtkWidget *) g_object_get_data (G_OBJECT (dialog), "entry_min");
gtk_entry_set_text (GTK_ENTRY (entry), "");
entry = (GtkWidget *) g_object_get_data (G_OBJECT (dialog), "entry_max");
gtk_entry_set_text (GTK_ENTRY (entry), "");

/*********************
 * show current list *
 *********************/

static_model (gamgi->atoms, window);
}

static void static_merge (gamgi_dlist *start)
{
gamgi_dlist *old, *new, *dlist, *append;

/**************************************************************
 * merge list Current (new) with list Before (old): append to *
 * list new all objects in list old that are not in list new  *
 **************************************************************/

dlist = append = NULL;

/***********************************
 * list new is empty: copy to list *
 * new all objects in list old     *
 ***********************************/

if (start == NULL)
  {
  for (old = atoms_old; old != NULL; old = old->next)
    {
    if (dlist == NULL)
      gamgi->atoms = dlist = gamgi_engine_dlist_add_end (NULL);
    else
      dlist = gamgi_engine_dlist_add_end (dlist);
    dlist->data = old->data;
    }
  return;
  }

/*****************************************
 * list new is not empty: append to list *
 * new different objects in list old     *
 *****************************************/

for (old = atoms_old; old != NULL; old = old->next)
  {
  for (new = start; new != append; new = new->next)
    {
    if (new->data == old->data) break;

    if (new->next == append)
      {
      if (append == NULL)
        append = dlist = gamgi_engine_dlist_add_end (new);
      else
        dlist = gamgi_engine_dlist_add_end (dlist);
      dlist->data = old->data;
      }
    }
  }

}

static gamgi_bool static_ok_double (double lower, double upper,
double *min, double *max, gamgi_window *window)
{
GtkWidget *dialog = window->dialog0;
GtkWidget *entry;
const char *name;
double swap;

/********************
 * get range limits *
 ********************/

*min = lower;
entry = (GtkWidget *) g_object_get_data (G_OBJECT (dialog), "entry_min");
name = gtk_entry_get_text (GTK_ENTRY (entry));
if (gamgi_io_token_check (name) == TRUE && gamgi_io_token_double_scan (name,
min, lower, upper) == FALSE)
  {
  gamgi_gtk_dialog_message_create ("Error", "Invalid range limits", window);
  return FALSE;
  }

*max = upper;
entry = (GtkWidget *) g_object_get_data (G_OBJECT (dialog), "entry_max");
name = gtk_entry_get_text (GTK_ENTRY (entry));
if (gamgi_io_token_check (name) == TRUE && gamgi_io_token_double_scan (name,
max, lower, upper) == FALSE)
  {
  gamgi_gtk_dialog_message_create ("Error", "Invalid range limits", window);
  return FALSE;
  }

if (*min > *max)
  {
  gamgi_gtk_dialog_message_create ("Warning", "Swapped range limits", window);

  swap = *min;
  *min = *max;
  *max = swap;
  }

return TRUE;
}

static gamgi_bool static_ok_float (float lower, float upper,
float *min, float *max, gamgi_window *window)
{
GtkWidget *dialog = window->dialog0;
GtkWidget *entry;
const char *name;
float swap;

/********************
 * get range limits *
 ********************/

*min = lower;
entry = (GtkWidget *) g_object_get_data (G_OBJECT (dialog), "entry_min");
name = gtk_entry_get_text (GTK_ENTRY (entry));
if (gamgi_io_token_check (name) == TRUE && gamgi_io_token_float_scan (name,
min, lower, upper) == FALSE)
  {
  gamgi_gtk_dialog_message_create ("Error", "Invalid range limits", window);
  return FALSE;
  }

*max = upper;
entry = (GtkWidget *) g_object_get_data (G_OBJECT (dialog), "entry_max");
name = gtk_entry_get_text (GTK_ENTRY (entry));
if (gamgi_io_token_check (name) == TRUE && gamgi_io_token_float_scan (name,
max, lower, upper) == FALSE)
  {
  gamgi_gtk_dialog_message_create ("Error", "Invalid range limits", window);
  return FALSE;
  }

if (*min > *max)
  {
  gamgi_gtk_dialog_message_create ("Warning", "Swapped range limits", window);

  swap = *min;
  *min = *max;
  *max = swap;
  }

return TRUE;
}

gamgi_dlist *static_ok_pick (gamgi_dlist *dlist_old,
gamgi_bool action, gamgi_window *window)
{
gamgi_object *object;
gamgi_dlist *dlist, *dlist_new;
GtkWidget *dialog = window->dialog0;
GtkWidget *treeview;
GtkTreeSelection *selection;
GtkTreeModel *model;
GtkTreeIter iter;
int number;

/*********************
 * get treeview data *
 *********************/

treeview = (GtkWidget *) g_object_get_data (G_OBJECT (dialog), "treeview");
selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview));
model = gtk_tree_view_get_model (GTK_TREE_VIEW (treeview));

/***************************************
 * when no objects have been selected, *
 * just include or exclude everything  *
 ***************************************/

dlist_new = NULL;

number = gtk_tree_selection_count_selected_rows (selection);
if (number == 0 && action == STATIC_ACTION_INCLUDE)
  {
  for (dlist = dlist_old; dlist != NULL; dlist = dlist->next)
    {
    dlist_new = gamgi_engine_dlist_add_start (dlist_new);
    dlist_new->data = dlist->data;
    }
  }

/*******************************************
 * otherwise traverse first level of model *
 * to build new list with selected objects *
 *******************************************/

if (number > 0)
  {
  gtk_tree_model_get_iter_first (model, &iter);
  do
    {
    if (gtk_tree_selection_iter_is_selected (selection, &iter) == TRUE)
      {
      if (action == STATIC_ACTION_INCLUDE)
        {
        dlist_new = gamgi_engine_dlist_add_start (dlist_new);
        gtk_tree_model_get (model, &iter, 1, &object, -1);
        dlist_new->data = object;
        }
      }
    else
      {
      if (action == STATIC_ACTION_EXCLUDE)
        {
        dlist_new = gamgi_engine_dlist_add_start (dlist_new);
        gtk_tree_model_get (model, &iter, 1, &object, -1);
        dlist_new->data = object;
        }
      }
    }
  while (gtk_tree_model_iter_next (model, &iter) == TRUE);
  }

return dlist_new;
}

gamgi_dlist *static_ok_element (gamgi_dlist *dlist_old,
gamgi_bool action, gamgi_window *window)
{
gamgi_atom *atom;
gamgi_dlist *dlist, *dlist_new;
GtkWidget *dialog = window->dialog0;
GtkWidget *entry;
char token1[GAMGI_ENGINE_TOKEN];
char token2[GAMGI_ENGINE_TOKEN];
const char *name;
int element1, element2, swap;

/********************
 * get range limits *
 ********************/

element1 = -INT_MAX;
entry = (GtkWidget *) g_object_get_data (G_OBJECT (dialog), "entry_min");
name = gtk_entry_get_text (GTK_ENTRY (entry));
if (gamgi_io_token_check (name) == TRUE &&
(gamgi_io_token_alpha_scan (name, token1, GAMGI_IO_TEXT, 
GAMGI_ENGINE_TOKEN) == FALSE || gamgi_chem_atom_number (token1,
&element1) == FALSE))
  {
  gamgi_gtk_dialog_message_create ("Error", "Invalid range limits", window);
  return NULL;
  }

element2 = INT_MAX;
entry = (GtkWidget *) g_object_get_data (G_OBJECT (dialog), "entry_max");
name = gtk_entry_get_text (GTK_ENTRY (entry));
if (gamgi_io_token_check (name) == TRUE &&
(gamgi_io_token_alpha_scan (name, token2, GAMGI_IO_TEXT,
GAMGI_ENGINE_TOKEN) == FALSE || gamgi_chem_atom_number (token2,
&element2) == FALSE))
  {
  gamgi_gtk_dialog_message_create ("Error", "Invalid range limits", window);
  return NULL;
  }

if (element1 > element2)
  {
  gamgi_gtk_dialog_message_create ("Warning", "Swapped range limits", window);

  swap = element1;
  element1 = element2;
  element2 = swap;
  }

/******************************
 * get list of selected atoms *
 ******************************/

dlist_new = NULL;
for (dlist = dlist_old; dlist != NULL; dlist = dlist->next)
  {
  atom = GAMGI_CAST_ATOM dlist->data;
  if (atom->element >= element1 && atom->element <= element2)
    {
    if (action == STATIC_ACTION_INCLUDE)
      {
      dlist_new = gamgi_engine_dlist_add_start (dlist_new);
      dlist_new->data = atom;
      }
    }
  else
    {
    if (action == STATIC_ACTION_EXCLUDE)
      {
      dlist_new = gamgi_engine_dlist_add_start (dlist_new);
      dlist_new->data = atom;
      }
    }
  }
  
return dlist_new;
}

gamgi_dlist *static_ok_type (gamgi_dlist *dlist_old,
gamgi_bool action, gamgi_window *window)
{
gamgi_atom *atom;
gamgi_dlist *dlist, *dlist_new;
GtkWidget *dialog = window->dialog0;
GtkWidget *entry;
char type1[GAMGI_ENGINE_TOKEN] = "";
char type2[GAMGI_ENGINE_TOKEN] = "";
char swap[GAMGI_ENGINE_TOKEN] = "";
const char *name;

/********************
 * get range limits *
 ********************/

entry = (GtkWidget *) g_object_get_data (G_OBJECT (dialog), "entry_min");
name = gtk_entry_get_text (GTK_ENTRY (entry));
if (gamgi_io_token_check (name) == TRUE && gamgi_io_token_alpha_scan (name,
type1, GAMGI_IO_TEXT, GAMGI_ENGINE_TOKEN) == FALSE)
  {
  gamgi_gtk_dialog_message_create ("Error", "Invalid range limits", window);
  return NULL;
  }

entry = (GtkWidget *) g_object_get_data (G_OBJECT (dialog), "entry_max");
name = gtk_entry_get_text (GTK_ENTRY (entry));
if (gamgi_io_token_check (name) == TRUE && gamgi_io_token_alpha_scan (name,
type2, GAMGI_IO_TEXT, GAMGI_ENGINE_TOKEN) == FALSE)
  {
  gamgi_gtk_dialog_message_create ("Error", "Invalid range limits", window);
  return NULL;
  }

if (*type1 != '\0' && *type2 != '\0' && strcmp (type1, type2) > 0)
  {
  gamgi_gtk_dialog_message_create ("Warning", "Swapped range limits", window);

  strcpy (swap, type1);
  strcpy (type1, type2);
  strcpy (type2, swap);
  }

/******************************
 * get list of selected atoms *
 ******************************/

dlist_new = NULL;
for (dlist = dlist_old; dlist != NULL; dlist = dlist->next)
  {
  atom = GAMGI_CAST_ATOM dlist->data;
  if ((*type1 == '\0' || strcmp (atom->object.type, type1) >= 0) &&
  (*type2 == '\0' || strcmp (atom->object.type, type2) <= 0))
    {
    if (action == STATIC_ACTION_INCLUDE)
      {
      dlist_new = gamgi_engine_dlist_add_start (dlist_new);
      dlist_new->data = atom;
      }
    }
  else
    {
    if (action == STATIC_ACTION_EXCLUDE)
      {
      dlist_new = gamgi_engine_dlist_add_start (dlist_new);
      dlist_new->data = atom;
      }
    }
  }

return dlist_new;
}

gamgi_dlist *static_ok_mass (gamgi_dlist *dlist_old,
gamgi_bool action, gamgi_window *window)
{
gamgi_atom *atom;
gamgi_dlist *dlist, *dlist_new;
gamgi_bool condition;
double min, max;

/********************
 * get range limits *
 ********************/

if (static_ok_double (-DBL_MAX, DBL_MAX,
&min, &max, window) == FALSE) return NULL;

/******************************
 * get list of selected atoms *
 ******************************/

dlist_new = NULL;
for (dlist = dlist_old; dlist != NULL; dlist = dlist->next)
  {
  atom = GAMGI_CAST_ATOM dlist->data;
  condition = (atom->mass >= min && atom->mass <= max);

  if ((condition == TRUE && action == STATIC_ACTION_INCLUDE) ||
  (condition == FALSE && action == STATIC_ACTION_EXCLUDE))
    {
    dlist_new = gamgi_engine_dlist_add_start (dlist_new);
    dlist_new->data = atom;
    }
  }

return dlist_new;
}

gamgi_dlist *static_ok_radius (gamgi_dlist *dlist_old,
gamgi_bool action, gamgi_window *window)
{
gamgi_atom *atom;
gamgi_dlist *dlist, *dlist_new;
gamgi_bool condition;
double min, max;

/********************
 * get range limits *
 ********************/

if (static_ok_double (-DBL_MAX, DBL_MAX,
&min, &max, window) == FALSE) return NULL;

/******************************
 * get list of selected atoms *
 ******************************/

dlist_new = NULL;
for (dlist = dlist_old; dlist != NULL; dlist = dlist->next)
  {
  atom = GAMGI_CAST_ATOM dlist->data;
  condition = (atom->radius >= min && atom->radius <= max);

  if ((condition == TRUE && action == STATIC_ACTION_INCLUDE) ||
  (condition == FALSE && action == STATIC_ACTION_EXCLUDE))
    {
    dlist_new = gamgi_engine_dlist_add_start (dlist_new);
    dlist_new->data = atom;
    }
  }

return dlist_new;
}

gamgi_dlist *static_ok_charge (gamgi_dlist *dlist_old,
gamgi_bool action, gamgi_window *window)
{
gamgi_atom *atom;
gamgi_dlist *dlist, *dlist_new;
gamgi_bool condition;
double min, max;

/********************
 * get range limits *
 ********************/

if (static_ok_double (-DBL_MAX, DBL_MAX,
&min, &max, window) == FALSE) return NULL;

/******************************
 * get list of selected atoms *
 ******************************/

dlist_new = NULL;
for (dlist = dlist_old; dlist != NULL; dlist = dlist->next)
  {
  atom = GAMGI_CAST_ATOM dlist->data;
  condition = (atom->charge >= min && atom->charge <= max);

  if ((condition == TRUE && action == STATIC_ACTION_INCLUDE) ||
  (condition == FALSE && action == STATIC_ACTION_EXCLUDE))
    {
    dlist_new = gamgi_engine_dlist_add_start (dlist_new);
    dlist_new->data = atom;
    }
  }

return dlist_new;
}

gamgi_dlist *static_ok_temperature (gamgi_dlist *dlist_old,
gamgi_bool action, gamgi_window *window)
{
gamgi_atom *atom;
gamgi_dlist *dlist, *dlist_new;
gamgi_bool condition;
double min, max;

/********************
 * get range limits *
 ********************/

if (static_ok_double (-DBL_MAX, DBL_MAX,
&min, &max, window) == FALSE) return NULL;

/******************************
 * get list of selected atoms *
 ******************************/

dlist_new = NULL;
for (dlist = dlist_old; dlist != NULL; dlist = dlist->next)
  {
  atom = GAMGI_CAST_ATOM dlist->data;
  condition = (atom->temperature >= min && atom->temperature <= max);

  if ((condition == TRUE && action == STATIC_ACTION_INCLUDE) ||
  (condition == FALSE && action == STATIC_ACTION_EXCLUDE))
    {
    dlist_new = gamgi_engine_dlist_add_start (dlist_new);
    dlist_new->data = atom;
    }
  }

return dlist_new;
}

gamgi_dlist *static_ok_occupancy (gamgi_dlist *dlist_old,
gamgi_bool action, gamgi_window *window)
{
gamgi_atom *atom;
gamgi_dlist *dlist, *dlist_new;
gamgi_bool condition;
double min, max;

/********************
 * get range limits *
 ********************/

if (static_ok_double (-DBL_MAX, DBL_MAX,
&min, &max, window) == FALSE) return NULL;

/******************************
 * get list of selected atoms *
 ******************************/

dlist_new = NULL;
for (dlist = dlist_old; dlist != NULL; dlist = dlist->next)
  {
  atom = GAMGI_CAST_ATOM dlist->data;
  condition = (atom->occupancy >= min && atom->occupancy <= max);

  if ((condition == TRUE && action == STATIC_ACTION_INCLUDE) ||
  (condition == FALSE && action == STATIC_ACTION_EXCLUDE))
    {
    dlist_new = gamgi_engine_dlist_add_start (dlist_new);
    dlist_new->data = atom;
    }
  }

return dlist_new;
}

gamgi_dlist *static_ok_x (gamgi_dlist *dlist_old,
gamgi_bool action, gamgi_window *window)
{
gamgi_atom *atom;
gamgi_dlist *dlist, *dlist_new;
gamgi_bool condition;
double min, max;

/********************
 * get range limits *
 ********************/

if (static_ok_double (-DBL_MAX, DBL_MAX,
&min, &max, window) == FALSE) return NULL;

/******************************
 * get list of selected atoms *
 ******************************/

dlist_new = NULL;
for (dlist = dlist_old; dlist != NULL; dlist = dlist->next)
  {
  atom = GAMGI_CAST_ATOM dlist->data;
  condition = (atom->position[0] >= min && atom->position[0] <= max);

  if ((condition == TRUE && action == STATIC_ACTION_INCLUDE) ||
  (condition == FALSE && action == STATIC_ACTION_EXCLUDE))
    {
    dlist_new = gamgi_engine_dlist_add_start (dlist_new);
    dlist_new->data = atom;
    }
  }

return dlist_new;
}

gamgi_dlist *static_ok_y (gamgi_dlist *dlist_old,
gamgi_bool action, gamgi_window *window)
{
gamgi_atom *atom;
gamgi_dlist *dlist, *dlist_new;
gamgi_bool condition;
double min, max;

/********************
 * get range limits *
 ********************/

if (static_ok_double (-DBL_MAX, DBL_MAX,
&min, &max, window) == FALSE) return NULL;

/******************************
 * get list of selected atoms *
 ******************************/

dlist_new = NULL;
for (dlist = dlist_old; dlist != NULL; dlist = dlist->next)
  {
  atom = GAMGI_CAST_ATOM dlist->data;
  condition = (atom->position[1] >= min && atom->position[1] <= max);

  if ((condition == TRUE && action == STATIC_ACTION_INCLUDE) ||
  (condition == FALSE && action == STATIC_ACTION_EXCLUDE))
    {
    dlist_new = gamgi_engine_dlist_add_start (dlist_new);
    dlist_new->data = atom;
    }
  }

return dlist_new;
}

gamgi_dlist *static_ok_z (gamgi_dlist *dlist_old,
gamgi_bool action, gamgi_window *window)
{
gamgi_atom *atom;
gamgi_dlist *dlist, *dlist_new;
gamgi_bool condition;
double min, max;

/********************
 * get range limits *
 ********************/

if (static_ok_double (-DBL_MAX, DBL_MAX,
&min, &max, window) == FALSE) return NULL;

/******************************
 * get list of selected atoms *
 ******************************/

dlist_new = NULL;
for (dlist = dlist_old; dlist != NULL; dlist = dlist->next)
  {
  atom = GAMGI_CAST_ATOM dlist->data;
  condition = (atom->position[2] >= min && atom->position[2] <= max);

  if ((condition == TRUE && action == STATIC_ACTION_INCLUDE) ||
  (condition == FALSE && action == STATIC_ACTION_EXCLUDE))
    {
    dlist_new = gamgi_engine_dlist_add_start (dlist_new);
    dlist_new->data = atom;
    }
  }

return dlist_new;
}

gamgi_dlist *static_ok_r (gamgi_dlist *dlist_old,
gamgi_bool action, gamgi_window *window)
{
gamgi_atom *atom;
gamgi_dlist *dlist, *dlist_new;
gamgi_bool condition;
float min, max;

/********************
 * get range limits *
 ********************/

if (static_ok_float (-FLT_MAX, FLT_MAX,
&min, &max, window) == FALSE) return NULL;

/******************************
 * get list of selected atoms *
 ******************************/

dlist_new = NULL;
for (dlist = dlist_old; dlist != NULL; dlist = dlist->next)
  {
  atom = GAMGI_CAST_ATOM dlist->data;
  condition = (atom->red >= min && atom->red <= max);

  if ((condition == TRUE && action == STATIC_ACTION_INCLUDE) ||
  (condition == FALSE && action == STATIC_ACTION_EXCLUDE))
    {
    dlist_new = gamgi_engine_dlist_add_start (dlist_new);
    dlist_new->data = atom;
    }
  }

return dlist_new;
}

gamgi_dlist *static_ok_g (gamgi_dlist *dlist_old,
gamgi_bool action, gamgi_window *window)
{
gamgi_atom *atom;
gamgi_dlist *dlist, *dlist_new;
gamgi_bool condition;
float min, max;

/********************
 * get range limits * 
 ********************/

if (static_ok_float (-FLT_MAX, FLT_MAX,
&min, &max, window) == FALSE) return NULL;

/******************************
 * get list of selected atoms *
 ******************************/

dlist_new = NULL;
for (dlist = dlist_old; dlist != NULL; dlist = dlist->next)
  {
  atom = GAMGI_CAST_ATOM dlist->data;
  condition = (atom->green >= min && atom->green <= max);

  if ((condition == TRUE && action == STATIC_ACTION_INCLUDE) ||
  (condition == FALSE && action == STATIC_ACTION_EXCLUDE))
    {
    dlist_new = gamgi_engine_dlist_add_start (dlist_new);
    dlist_new->data = atom;
    }
  }

return dlist_new;
}

gamgi_dlist *static_ok_b (gamgi_dlist *dlist_old,
gamgi_bool action, gamgi_window *window)
{
gamgi_atom *atom;
gamgi_dlist *dlist, *dlist_new;
gamgi_bool condition;
float min, max;

/********************
 * get range limits * 
 ********************/

if (static_ok_float (-FLT_MAX, FLT_MAX,
&min, &max, window) == FALSE) return NULL;

/******************************
 * get list of selected atoms *
 ******************************/

dlist_new = NULL;
for (dlist = dlist_old; dlist != NULL; dlist = dlist->next)
  {
  atom = GAMGI_CAST_ATOM dlist->data;
  condition = (atom->blue >= min && atom->blue <= max);

  if ((condition == TRUE && action == STATIC_ACTION_INCLUDE) ||
  (condition == FALSE && action == STATIC_ACTION_EXCLUDE))
    {
    dlist_new = gamgi_engine_dlist_add_start (dlist_new);
    dlist_new->data = atom;
    }
  }

return dlist_new;
}

gamgi_dlist *static_ok_size (gamgi_dlist *dlist_old,
gamgi_bool action, gamgi_window *window)
{
gamgi_atom *atom;
gamgi_dlist *dlist, *dlist_new;
gamgi_bool condition;
float min, max;

/********************
 * get range limits *
 ********************/

if (static_ok_float (-FLT_MAX, FLT_MAX,
&min, &max, window) == FALSE) return NULL;

/******************************
 * get list of selected atoms *
 ******************************/

dlist_new = NULL;
for (dlist = dlist_old; dlist != NULL; dlist = dlist->next)
  {
  atom = GAMGI_CAST_ATOM dlist->data;
  condition = (atom->size >= min && atom->size <= max);

  if ((condition == TRUE && action == STATIC_ACTION_INCLUDE) ||
  (condition == FALSE && action == STATIC_ACTION_EXCLUDE))
    {
    dlist_new = gamgi_engine_dlist_add_start (dlist_new);
    dlist_new->data = atom;
    }
  }

return dlist_new;
}

gamgi_dlist *static_ok_variancy (gamgi_dlist *dlist_old,
gamgi_bool action, gamgi_window *window)
{
gamgi_atom *atom;
gamgi_dlist *dlist, *dlist_new;
gamgi_bool condition;
float min, max;

/********************
 * get range limits *
 ********************/

if (static_ok_float (-FLT_MAX, FLT_MAX,
&min, &max, window) == FALSE) return NULL;

/******************************
 * get list of selected atoms *
 ******************************/

dlist_new = NULL;
for (dlist = dlist_old; dlist != NULL; dlist = dlist->next)
  {
  atom = GAMGI_CAST_ATOM dlist->data;
  condition = (atom->variancy >= min && atom->variancy <= max);

  if ((condition == TRUE && action == STATIC_ACTION_INCLUDE) ||
  (condition == FALSE && action == STATIC_ACTION_EXCLUDE))
    {
    dlist_new = gamgi_engine_dlist_add_start (dlist_new);
    dlist_new->data = atom;
    }
  }

return dlist_new;
}

gamgi_dlist *static_ok_scale (gamgi_dlist *dlist_old,
gamgi_bool action, gamgi_window *window)
{
gamgi_atom *atom;
gamgi_dlist *dlist, *dlist_new;
gamgi_bool condition;
double min, max;

/********************
 * get range limits *
 ********************/

if (static_ok_double (-DBL_MAX, DBL_MAX,
&min, &max, window) == FALSE) return NULL;

/******************************
 * get list of selected atoms *
 ******************************/

dlist_new = NULL;
for (dlist = dlist_old; dlist != NULL; dlist = dlist->next)
  {
  atom = GAMGI_CAST_ATOM dlist->data;
  condition = (atom->object.scale >= min && atom->object.scale <= max);

  if ((condition == TRUE && action == STATIC_ACTION_INCLUDE) ||
  (condition == FALSE && action == STATIC_ACTION_EXCLUDE))
    {
    dlist_new = gamgi_engine_dlist_add_start (dlist_new);
    dlist_new->data = atom;
    }
  }

return dlist_new;
}

static void static_ok (GtkWidget *widget, void *data)
{
gamgi_window *window = GAMGI_CAST_WINDOW data;
gamgi_dlist *dlist_new, *dlist_old;
GtkWidget *dialog = window->dialog0;
GtkWidget *combo;
int scope, action, method;

combo = (GtkWidget *) g_object_get_data (G_OBJECT (dialog), "combo_scope");
scope = gtk_combo_box_get_active (GTK_COMBO_BOX (combo));

combo = (GtkWidget *) g_object_get_data (G_OBJECT (dialog), "combo_action");
action = gtk_combo_box_get_active (GTK_COMBO_BOX (combo));

combo = (GtkWidget *) g_object_get_data (G_OBJECT (dialog), "combo_method");
method = gtk_combo_box_get_active (GTK_COMBO_BOX (combo));

/************************************************************
 * build list of candidate atoms, according to chosen scope *
 *                                                          *
 * build list of selected atoms, according to chosen method *
 ************************************************************/

dlist_old = static_dlist (scope, window);

switch (method)
  {
  case STATIC_METHOD_PICK:
  dlist_new = static_ok_pick (dlist_old, action, window);
  break;

  case STATIC_METHOD_ELEMENT:
  dlist_new = static_ok_element (dlist_old, action, window);
  break;

  case STATIC_METHOD_TYPE:
  dlist_new = static_ok_type (dlist_old, action, window);
  break;

  case STATIC_METHOD_MASS:
  dlist_new = static_ok_mass (dlist_old, action, window);
  break;

  case STATIC_METHOD_RADIUS:
  dlist_new = static_ok_radius (dlist_old, action, window);
  break;

  case STATIC_METHOD_CHARGE:
  dlist_new = static_ok_charge (dlist_old, action, window);
  break;

  case STATIC_METHOD_TEMPERATURE:
  dlist_new = static_ok_temperature (dlist_old, action, window);
  break;

  case STATIC_METHOD_OCCUPANCY:
  dlist_new = static_ok_occupancy (dlist_old, action, window);
  break;

  case STATIC_METHOD_X:
  dlist_new = static_ok_x (dlist_old, action, window);
  break;

  case STATIC_METHOD_Y:
  dlist_new = static_ok_y (dlist_old, action, window);
  break;

  case STATIC_METHOD_Z:
  dlist_new = static_ok_z (dlist_old, action, window);
  break;

  case STATIC_METHOD_R:
  dlist_new = static_ok_r (dlist_old, action, window);
  break;

  case STATIC_METHOD_G:
  dlist_new = static_ok_g (dlist_old, action, window);
  break;

  case STATIC_METHOD_B:
  dlist_new = static_ok_b (dlist_old, action, window);
  break;

  case STATIC_METHOD_SIZE:
  dlist_new = static_ok_size (dlist_old, action, window);
  break;

  case STATIC_METHOD_VARIANCY:
  dlist_new = static_ok_variancy (dlist_old, action, window);
  break;

  case STATIC_METHOD_SCALE:
  dlist_new = static_ok_scale (dlist_old, action, window);
  break;

  default:
  dlist_new = NULL;
  }

/****************************************
 * invert list: first node becomes last *
 ****************************************/

dlist_new = gamgi_engine_dlist_invert (dlist_new);

/********************************************
 * remove dlist_old, except for list Before *
 * as this is deleted after, and list       *
 * Current as this will become list Before  *
 *                                          *
 * remove list Before, the list Current     *
 * becomes the new list Before              *
 *                                          *
 * The selected list replaces list Current  *
 * when Scope is Current and merges with    *
 * list Current when Scope is different     *
 ********************************************/

if (dlist_old != atoms_old && dlist_old != gamgi->atoms)
  gamgi_engine_dlist_remove_all (dlist_old);

gamgi_engine_dlist_remove_all (atoms_old);
atoms_old = gamgi->atoms;

gamgi->atoms = dlist_new;
if (scope != STATIC_SCOPE_CURRENT)
  static_merge (dlist_new);

/*****************************************
 * Change color for Atom label to signal *
 * that a list of selected atoms exists  *
 *                                       *
 * reset dialog: the Scope list is set   *
 * to Current and the model is updated   *
 *****************************************/

if (gamgi->atoms == NULL)
  gamgi_global_selection_hide ("button_atom");
else
  gamgi_global_selection_show ("button_atom");

static_reset (window);
}

static void static_init (gamgi_window *window)
{
GtkWidget *dialog = window->dialog0;
GtkWidget *combo;

/**************************************************
 * everytime the dialog is created: initialize    *
 * list Before and unlink objects in list Current *
 **************************************************/

atoms_old = NULL;
gamgi_global_selection_unlink (gamgi->atoms);

/************************************
 * initialize Scope, Action, Method *
 ************************************/

combo = (GtkWidget *) g_object_get_data (G_OBJECT (dialog), "combo_scope");
if (gamgi->atoms == NULL)
  gtk_combo_box_set_active (GTK_COMBO_BOX (combo), STATIC_SCOPE_LAYER);
else
  gtk_combo_box_set_active (GTK_COMBO_BOX (combo), STATIC_SCOPE_CURRENT);

combo = (GtkWidget *) g_object_get_data (G_OBJECT (dialog), "combo_action");
gtk_combo_box_set_active (GTK_COMBO_BOX (combo), STATIC_ACTION_INCLUDE);

combo = (GtkWidget *) g_object_get_data (G_OBJECT (dialog), "combo_method");
gtk_combo_box_set_active (GTK_COMBO_BOX (combo), STATIC_METHOD_PICK);
}

static void static_press (gamgi_object *object, void *data)
{
gamgi_window *window_dialog = gamgi->window_dialog;
gamgi_object *object_tree;
GtkWidget *dialog = window_dialog->dialog0;
GtkWidget *treeview;
GtkTreeSelection *selection;
GtkTreeModel *model;
GtkTreeIter iter;

/*********************
 * get treeview data *
 *********************/

treeview = (GtkWidget *) g_object_get_data (G_OBJECT (dialog), "treeview");
selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview));
model = gtk_tree_view_get_model (GTK_TREE_VIEW (treeview));

/***************************************************************
 * a valid object has been found, check if the object is in    *
 * the GtkTreeView Current list: 1) yes, set the object as     *
 * selected in the list; 2) no, ignore the press button event. *
 ***************************************************************/

if (gtk_tree_model_get_iter_first (model, &iter) == FALSE) return;
do
  {
  gtk_tree_model_get (model, &iter, 1, &object_tree, -1);
  if (object == object_tree)
    { gtk_tree_selection_select_iter (selection, &iter); return; }
  }
  while (gtk_tree_model_iter_next (model, &iter) == TRUE);

}

void gamgi_gtk_atom_select_press (gamgi_window *window_mouse,    
GdkEventButton *event, int x, int y, gamgi_window *window_dialog)
{
/**************************************************
 * Ctrl key has not been pressed: this is a local *
 * object selection, to set new object with focus *
 **************************************************/

if ((event->state & GDK_CONTROL_MASK) == 0)
  {
  if (window_dialog == window_mouse)
    gamgi_mesa_select_object (window_mouse,
    x, y, GAMGI_ENGINE_ATOM, FALSE, NULL);
  }

/*********************************************************
 * Ctrl key has been pressed: this is a global object    *
 * selection, to find an object belonging to GtkTreeView *
 * Current list, to mark the object as selected          *
 *********************************************************/

else
  gamgi_mesa_select_object (window_mouse,
  x, y, GAMGI_ENGINE_ATOM, TRUE, static_press);
}

void gamgi_gtk_atom_select (GtkWidget *widget, void *data)
{
gamgi_window *window = GAMGI_CAST_WINDOW data;
GtkWidget *dialog;
GtkWidget *notebook;
GtkWidget *scrolled_window;
GtkWidget *frame;
GtkWidget *treeview;
GtkWidget *combo;
GtkWidget *button;
GtkWidget *entry;
GtkWidget *label;
GtkWidget *hbox_center, *hbox_left;
GtkWidget *vbox_dialog, *vbox_page, *vbox_center, *vbox_top, *vbox_top_top;
GtkCellRenderer *renderer;
GtkTreeViewColumn *column;
GtkTreeSelection *selection;
GtkListStore *store;
GtkTreeIter iter;

/******************
 * dialog level 0 *
 ******************/

dialog = gamgi_gtk_dialog_task0_create ("Atom Select", window);
window->action = GAMGI_GTK_ATOM_SELECT;
gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
g_signal_connect (dialog, "destroy", G_CALLBACK (static_destroy), window);

/********************
 * global container *
 ********************/

vbox_dialog = gtk_vbox_new (FALSE, 5);
gtk_container_add (GTK_CONTAINER (dialog), vbox_dialog);
gtk_widget_show (vbox_dialog);

/************
 * notebook *
 ************/

notebook = gtk_notebook_new ();
gtk_box_pack_start (GTK_BOX (vbox_dialog), notebook, FALSE, FALSE, 0);
gtk_notebook_set_tab_pos (GTK_NOTEBOOK (notebook), GTK_POS_TOP);
g_object_set_data (G_OBJECT (dialog), "notebook", notebook);
gtk_widget_show (notebook);

/****************
 * Objects page *
 ****************/

vbox_page = gtk_vbox_new (FALSE, 5);
label = gtk_label_new (" List ");
gtk_notebook_append_page (GTK_NOTEBOOK (notebook), vbox_page, label);
gtk_container_set_border_width (GTK_CONTAINER (vbox_page), 10);
gtk_widget_show (vbox_page);

vbox_center = gtk_vbox_new (TRUE, 0);
gtk_box_pack_start (GTK_BOX (vbox_page), vbox_center, TRUE, TRUE, 0);
gtk_widget_show (vbox_center);

hbox_left = gtk_hbox_new (FALSE, 10);
gtk_box_pack_start (GTK_BOX (vbox_center), hbox_left, FALSE, FALSE, 0);
gtk_widget_show (hbox_left);

/*************
 * left side *
 *************/

/*******************
 * Object TreeView *
 *******************/

frame = gtk_frame_new (NULL);
gtk_box_pack_start (GTK_BOX (hbox_left), frame, TRUE, TRUE, 0);
gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_ETCHED_IN);
gtk_widget_show (frame);

scrolled_window = gtk_scrolled_window_new (NULL, NULL);
gtk_container_add (GTK_CONTAINER (frame), scrolled_window);
gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
gtk_widget_show (scrolled_window);

treeview = gtk_tree_view_new ();
gtk_container_add (GTK_CONTAINER (scrolled_window), treeview);
gtk_widget_set_size_request (treeview, 250, 400);
selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview));
gtk_tree_selection_set_mode (selection, GTK_SELECTION_MULTIPLE);
g_object_set_data (G_OBJECT (dialog), "treeview", treeview);
gtk_widget_show (treeview);

renderer = gtk_cell_renderer_text_new ();
column = gtk_tree_view_column_new_with_attributes ("List", renderer, "text", 0, NULL);
gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);
gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (treeview), FALSE);

/**************
 * right side *
 **************/

vbox_center = gtk_vbox_new (TRUE, 0);
gtk_box_pack_start (GTK_BOX (hbox_left), vbox_center, FALSE, FALSE, 0);
gtk_widget_show (vbox_center);

vbox_top = gtk_vbox_new (FALSE, 15);
gtk_box_pack_start (GTK_BOX (vbox_center), vbox_top, FALSE, FALSE, 0);
gtk_widget_show (vbox_top);

/*********
 * Scope *
 *********/

vbox_top_top = gtk_vbox_new (FALSE, 5);
gtk_box_pack_start (GTK_BOX (vbox_top), vbox_top_top, FALSE, FALSE, 0);
gtk_widget_show (vbox_top_top);

label = gtk_label_new ("Scope");
gtk_box_pack_start (GTK_BOX (vbox_top_top), label, FALSE, FALSE, 0);
gtk_widget_show (label);

combo = gtk_combo_box_new ();
gtk_box_pack_start (GTK_BOX (vbox_top_top), combo, FALSE, FALSE, 0);
g_signal_connect (combo, "changed",
G_CALLBACK (static_scope), window);
g_object_set_data (G_OBJECT (dialog), "combo_scope", combo);
gtk_widget_show (combo);

store = gtk_list_store_new (1, G_TYPE_STRING);
gtk_list_store_append (store, &iter);
gtk_list_store_set (store, &iter, 0, "Gamgi", -1);
gtk_list_store_append (store, &iter);
gtk_list_store_set (store, &iter, 0, "Window", -1);
gtk_list_store_append (store, &iter);
gtk_list_store_set (store, &iter, 0, "Layer", -1);
gtk_list_store_append (store, &iter);
gtk_list_store_set (store, &iter, 0, "Before", -1);
gtk_list_store_append (store, &iter);
gtk_list_store_set (store, &iter, 0, "Current", -1);
gtk_combo_box_set_model (GTK_COMBO_BOX (combo), GTK_TREE_MODEL (store));
g_object_unref (store);

renderer = gtk_cell_renderer_text_new ();
gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (combo), renderer, FALSE);
gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (combo), renderer, "text", 0, NULL);

/**********
 * Action *
 **********/

vbox_top_top = gtk_vbox_new (FALSE, 5);
gtk_box_pack_start (GTK_BOX (vbox_top), vbox_top_top, FALSE, FALSE, 0);
gtk_widget_show (vbox_top_top);

label = gtk_label_new ("Action");
gtk_box_pack_start (GTK_BOX (vbox_top_top), label, FALSE, FALSE, 0);
gtk_widget_show (label);

combo = gtk_combo_box_new ();
gtk_box_pack_start (GTK_BOX (vbox_top_top), combo, FALSE, FALSE, 0);
g_object_set_data (G_OBJECT (dialog), "combo_action", combo);
gtk_widget_show (combo);

store = gtk_list_store_new (1, G_TYPE_STRING);
gtk_list_store_append (store, &iter);
gtk_list_store_set (store, &iter, 0, "Include", -1);
gtk_list_store_append (store, &iter);
gtk_list_store_set (store, &iter, 0, "Exclude", -1);
gtk_combo_box_set_model (GTK_COMBO_BOX (combo), GTK_TREE_MODEL (store));
g_object_unref (store);

renderer = gtk_cell_renderer_text_new ();
gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (combo), renderer, FALSE);
gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (combo), renderer, "text", 0, NULL);

/************
 * Function *
 ************/

vbox_top_top = gtk_vbox_new (FALSE, 5);
gtk_box_pack_start (GTK_BOX (vbox_top), vbox_top_top, FALSE, FALSE, 0);
gtk_widget_show (vbox_top_top);

label = gtk_label_new ("Method");
gtk_box_pack_start (GTK_BOX (vbox_top_top), label, FALSE, FALSE, 0);
gtk_widget_show (label);

combo = gtk_combo_box_new ();
gtk_box_pack_start (GTK_BOX (vbox_top_top), combo, FALSE, FALSE, 0);
g_object_set_data (G_OBJECT (dialog), "combo_method", combo);
gtk_widget_show (combo);

store = gtk_list_store_new (1, G_TYPE_STRING);
gtk_list_store_append (store, &iter);
gtk_list_store_set (store, &iter, 0, "Pick", -1);
gtk_list_store_append (store, &iter);
gtk_list_store_set (store, &iter, 0, "Element", -1);
gtk_list_store_append (store, &iter);
gtk_list_store_set (store, &iter, 0, "Type", -1);
gtk_list_store_append (store, &iter);
gtk_list_store_set (store, &iter, 0, "Mass", -1);
gtk_list_store_append (store, &iter);
gtk_list_store_set (store, &iter, 0, "Radius", -1);
gtk_list_store_append (store, &iter);
gtk_list_store_set (store, &iter, 0, "Charge", -1);
gtk_list_store_append (store, &iter);
gtk_list_store_set (store, &iter, 0, "Temperature", -1);
gtk_list_store_append (store, &iter);
gtk_list_store_set (store, &iter, 0, "Occupancy", -1);
gtk_list_store_append (store, &iter);
gtk_list_store_set (store, &iter, 0, "X", -1);
gtk_list_store_append (store, &iter);
gtk_list_store_set (store, &iter, 0, "Y", -1);
gtk_list_store_append (store, &iter);
gtk_list_store_set (store, &iter, 0, "Z", -1);
gtk_list_store_append (store, &iter);
gtk_list_store_set (store, &iter, 0, "R", -1);
gtk_list_store_append (store, &iter);
gtk_list_store_set (store, &iter, 0, "G", -1);
gtk_list_store_append (store, &iter);
gtk_list_store_set (store, &iter, 0, "B", -1);
gtk_list_store_append (store, &iter);
gtk_list_store_set (store, &iter, 0, "Size", -1);
gtk_list_store_append (store, &iter);
gtk_list_store_set (store, &iter, 0, "Variancy", -1);
gtk_list_store_append (store, &iter);
gtk_list_store_set (store, &iter, 0, "Scale", -1);

gtk_combo_box_set_model (GTK_COMBO_BOX (combo), GTK_TREE_MODEL (store));
g_object_unref (store);

renderer = gtk_cell_renderer_text_new ();
gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (combo), renderer, FALSE);
gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (combo), renderer, "text", 0, NULL);

/*********
 * Range *
 *********/

vbox_top_top = gtk_vbox_new (FALSE, 5);
gtk_box_pack_start (GTK_BOX (vbox_top), vbox_top_top, FALSE, FALSE, 0);
gtk_widget_show (vbox_top_top);

label = gtk_label_new ("Range");
gtk_box_pack_start (GTK_BOX (vbox_top_top), label, FALSE, FALSE, 0);
gtk_widget_show (label);

hbox_left = gtk_hbox_new (FALSE, 5);
gtk_box_pack_start (GTK_BOX (vbox_top_top), hbox_left, FALSE, FALSE, 0);
gtk_widget_show (hbox_left);

entry = gtk_entry_new ();
gtk_box_pack_end (GTK_BOX (hbox_left), entry, FALSE, FALSE, 0);
gtk_entry_set_width_chars (GTK_ENTRY (entry), GAMGI_GTK_CHAR_10);
gtk_entry_set_max_length (GTK_ENTRY (entry), GAMGI_GTK_CHAR_10);
g_object_set_data (G_OBJECT (dialog), "entry_min", entry);
gtk_widget_show (entry);

label = gtk_label_new ("Min");
gtk_box_pack_end (GTK_BOX (hbox_left), label, FALSE, FALSE, 0);
gtk_widget_show (label);

hbox_left = gtk_hbox_new (FALSE, 5);
gtk_box_pack_start (GTK_BOX (vbox_top_top), hbox_left, FALSE, FALSE, 0);
gtk_widget_show (hbox_left);

entry = gtk_entry_new ();
gtk_box_pack_end (GTK_BOX (hbox_left), entry, FALSE, FALSE, 0);
gtk_entry_set_width_chars (GTK_ENTRY (entry), GAMGI_GTK_CHAR_10);
gtk_entry_set_max_length (GTK_ENTRY (entry), GAMGI_GTK_CHAR_10);
g_object_set_data (G_OBJECT (dialog), "entry_max", entry);
gtk_widget_show (entry);

label = gtk_label_new ("Max");
gtk_box_pack_end (GTK_BOX (hbox_left), label, FALSE, FALSE, 0);
gtk_widget_show (label);

/*********************
 * Ok/Cancel buttons *
 *********************/

hbox_center = gtk_hbox_new (TRUE, 0);
gtk_box_pack_start (GTK_BOX (vbox_dialog), hbox_center, FALSE, FALSE, 0);
gtk_widget_show (hbox_center);

button = gamgi_gtk_dialog_button_create ("Ok", NULL);
gtk_box_pack_start (GTK_BOX (hbox_center), button, FALSE, FALSE, 0);
gtk_widget_set_size_request (button, GAMGI_GTK_BUTTON_WIDTH, -1);
GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
g_signal_connect (button, "clicked",
G_CALLBACK (static_ok), window);
gtk_widget_show (button);

button = gamgi_gtk_dialog_button_create ("Cancel", "red");
gtk_box_pack_start (GTK_BOX (hbox_center), button, FALSE, FALSE, 0);
gtk_widget_set_size_request (button, GAMGI_GTK_BUTTON_WIDTH, -1);
GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
gtk_widget_grab_focus (button);
g_signal_connect (button, "clicked",
G_CALLBACK (gamgi_gtk_dialog_task0_remove), window);
gtk_widget_show (button);

static_init (window);
gtk_widget_show (dialog);
}
